#include <cstdio>
#include <set>
#include <algorithm>
#include <iostream>
#include <vector>
#include <cstring>
#include <ctime>
using namespace std;
#define NAME "accounts"
const long long INF = 1e18 + 1;

long long safeMul(long long a, long long b) {
    long double xa = a;
    long double xb = b;
    if (xa * xb > INF)
        return INF;
    long long res = a * b;
    if (res >= INF)
        return INF;
    return res;
}

long long gcd(long long a, long long b) {
    while (a != 0) {
        b %= a;
        swap(a, b);
    }
    return b;
}

long long lcm(long long a, long long b) {
    long long g = gcd(a, b);
    a /= g;
    return safeMul(a, b);
}

struct Fract {
    long long p;
    long long q;
    bool overflow;

    Fract(): p(0), q(0), overflow(false) {}

    Fract(long long a, long long b): overflow(false) {
        long long g = gcd(a, b);
        p = a / g;
        q = b / g;
    }

    Fract operator * (const Fract& t) {
        long long g1 = gcd(p, t.q);
        long long g2 = gcd(q, t.p);
        long long p1 = safeMul(p / g1, t.p / g2);
        if (p1 == INF) {
            overflow = true;
            return *this;
        }
        long long q1 = safeMul(q / g2, t.q / g1);
        if (q == INF) {
            overflow = true;
            return *this;
        }
        return Fract(p1, q1);
    }

    bool operator == (const Fract& t) {
        return p == t.p && q == t.q;
    }
};

const int N = 1e5 + 10;
const int M = 3e5 + 10;

vector <pair <int, Fract> > g[N];
vector <int>  indexes[M];
vector <long long> values[M];
Fract ans[N];
bool fail;
long long b[N];

void dfs(int v, long long& lc, vector <int>& vert) {
    vert.push_back(v);
    lc = lcm(lc, ans[v].q);
    if (lc == INF)
        fail = true;

    if (fail)
        return;
    for (size_t i = 0; i < g[v].size(); ++i) {
        int to = g[v][i].first;
        Fract w = g[v][i].second;
        if (ans[to] == Fract()) {
            ans[to] = ans[v] * w;
            if (ans[to].overflow)
                fail = true;
            else
                dfs(to, lc, vert);
        } else {
            Fract cur = ans[v] * w;
            if (cur.overflow || !(cur == ans[to]))
                fail = true;
        }
    }
}

int main() {
	int l = clock();
    freopen(NAME".in", "r", stdin);
    freopen(NAME".out", "w", stdout);
    int n, m;
    cin >> n >> m;
    for (int i = 0; i < m; ++i) {
        int ki;
        scanf("%d", &ki);
        indexes[i] = vector <int>(ki);
        values[i] = vector <long long>(ki);

        for (int j = 0; j < ki; ++j) {
            int id;
            long long val;
            scanf("%d%I64d", &id, &val);
            --id;
            indexes[i][j] = id;
            values[i][j] = val;
        }
        int id = indexes[i][0];
        long long val = values[i][0];
        for (int j = 1; j < ki; ++j) {
            int to = indexes[i][j];
            g[id].push_back(make_pair(to, Fract(values[i][j], val)));
            g[to].push_back(make_pair(id, Fract(val, values[i][j])));
        }
    }

    for (int i = 0; i < n && !fail; ++i)
        if (ans[i] == Fract()) {
            ans[i] = Fract(1, 1);
            long long b1 = 1;
            vector <int> vert;
            dfs(i, b1, vert);
            if (fail)
                break;
            for (size_t j = 0; j < vert.size(); ++j) {
                int id = vert[j];
                long long p = ans[id].p;
                long long q = ans[id].q;
                b[id] = safeMul(p, b1 / q);
                if (b[id] == INF) {
                    fail = true;
                    break;
                }
            }
        }
    for (int i = 0; i < m && !fail; ++i) {
        long long t = 0;
        for (size_t j = 0; j < values[i].size() && !fail; ++j) {
            int id = indexes[i][j];
            if (values[i][j] % b[id] != 0) {
                fail = true;
                break;
            }
            if (j == 0)
                t = values[i][j] / b[id];
            else if (values[i][j] / b[id] != t)
                fail = true;
        }
    }

    if (fail) {
        cout << "NO";
        return 0;
    }
    cout << "YES\n";
    for (int i = 0; i < n; ++i)
        printf("%I64d ", b[i]);
	cerr << "time = " << (clock() - l) * 1.0 / CLOCKS_PER_SEC << endl;
    return 0;
}
