import java.io.*;
import java.math.BigInteger;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0xDEAD);
    public static final int MX_N = 100000;//10^5
    public static final int MX_M = 100000;//10^5
    public static final int MX_SUM_KI = 300000;
    public static final long MX_VAL = 1000000000000000000L;//10^18
    private static final long[] LIMITS = new long[] {20, 100000, MX_VAL};
    private static final int MAX_SMALL_PRIME = 10000000;//10^7
    private static final int MIN_BIG_PRIME = 100000000;
    private static final int NUM_BIG_PRIMES = 1000000;//10^6
    private static final int NUM_MANY_DIVS = 100000;
    private static final int NUM_PRIMES_FOR_GENERATING_MANY_DIVS = 10;
    private long CURRENT_MAXVAL_TMP;

    private long[] SMALL_PRIMES;
    private long[] BIG_PRIMES;
    private long[] MANY_DIVS;

    public static class Test {
        public int n;
        public int m;
        public ArrayList<ArrayList<Integer>> indexes = new ArrayList<>();
        public ArrayList<ArrayList<Long>> values = new ArrayList<>();

        public Test(int n) {
            this.n = n;
        }

        boolean[] use;
        private int sz;
        private ArrayList<ArrayList<Integer>> graph = new ArrayList<>();
        private void dfs(int id) {
            use[id] = true;
            sz++;
            for (int i = 0; i < graph.get(id).size(); ++i) {
                int to = graph.get(id).get(i);
                if (!use[to])
                    dfs(to);
            }
        }

        public int getNumComponents() {
            use = new boolean[n];
            for (int i = 0; i < n; ++i)
                graph.add(new ArrayList<Integer>());
            for (int i = 0; i < indexes.size(); ++i) {
                int from = indexes.get(i).get(0);
                for (int j = 1; j < indexes.get(i).size(); ++j) {
                    int to = indexes.get(i).get(j);
                    graph.get(from).add(to);
                    graph.get(to).add(from);
                }
            }

            int comp = 0;
            for (int i = 0; i < n; ++i)
                if (!use[i]) {
                    sz = 0;
                    dfs(i);
                    if (sz > 1)
                        comp++;
                }
            return comp;
        }

        public boolean ensureLimitsOnHints(long limitVal) {
            for (int i = 0; i < m; ++i) {
                for (int j = 0; j < values.get(i).size(); ++j) {

                    if (values.get(i).get(j) > limitVal) {
                        System.out.println(i + " " + j + " " + values.get(i).get(j));
                        return false;
                    }
                    if (values.get(i).get(j) <= 0) {
                        System.out.println("val = " + values.get(i).get(j));
                        return false;
                    }
                }
            }
            return true;
        }

        public void addHint() {
            ++m;
            indexes.add(new ArrayList<Integer>());
            values.add(new ArrayList<Long>());
        }

        public void addHint(int id, long val) {
            indexes.get(m - 1).add(id);
            values.get(m - 1).add(val);
        }

        public int getSumKi() {
            int ret = 0;
            for (int i = 0; i < m; ++i)
                ret += indexes.get(i).size();
            return ret;
        }

        public void incIndexes() {
            for (int i = 0; i < m; ++i)
                for (int j = 0; j < indexes.get(i).size(); ++j)
                    indexes.get(i).set(j, indexes.get(i).get(j) + 1);
        }
    }

    public static class Pair implements Comparable {
        public int key;
        public long value;
        Pair(int k, long v) {
            key = k;
            value = v;
        }

        @Override
        public int compareTo(Object o) {
            Pair oth = (Pair)o;
            if (key < oth.key || key == oth.key && value < oth.value)
                return -1;
            if (key == oth.key && value == oth.value)
                return 0;
            return 1;
        }
    }

    public void write(Test cur, String folder, String subtask, int points, String comment) throws IOException {
        if (!cur.ensureLimitsOnHints(LIMITS[subtaskCount]))
            throw new RuntimeException("Too big xj in subtask = " + subtask + " in test = " + (testCount + 1));

        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        int n = cur.n;
        int m = cur.m;

        /*int sumKi = cur.getSumKi();
        System.out.println("n = " + n + " m = " + m + " sumKi = " + sumKi + " comp = " + cur.getNumComponents());*/

        cur.incIndexes();
        out.println(n + " " + m);
        for (int i = 0; i < m; ++i) {
            int hints = cur.indexes.get(i).size();
            out.println(hints);
            for (int j = 0; j + 1 < hints; ++j)
                out.print(cur.indexes.get(i).get(j) + " " + cur.values.get(i).get(j) + " ");
            out.println(cur.indexes.get(i).get(hints - 1) + " " + cur.values.get(i).get(hints - 1));
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }

    public int upperBound(long[] arr, long rightBound) {
        int l = -1, r = arr.length, mid;
        while (l + 1 < r) {
            mid = (l + r) / 2;
            if (rightBound >= arr[mid])
                l = mid;
            else
                r = mid;
        }
        return l;
    }

    private long[] calcSmallPrimes(int maxVal) {
        boolean[] p = new boolean[maxVal + 1];
        for (int i = 2; i * i <= maxVal; ++i)
            if (!p[i])
                for (int j = i * i; j <= maxVal; j += i)
                    p[j] = true;
        List<Integer> primes = new ArrayList<>();
        for (int j = 2; j <= maxVal; ++j)
            if (!p[j])
                primes.add(j);
        long[] ret = new long[primes.size()];
        for (int i = 0; i < primes.size(); ++i)
            ret[i] = primes.get(i);
        return ret;
    }

    private long[] calcBigPrimes() {
        int sz = 0;
        long x = 100000000;//10^8
        final int T = 10;
        long[] ret = new long[NUM_BIG_PRIMES];
        for (int i = 0; i < T && x <= MX_VAL; ++i) {
            BigInteger cur = BigInteger.valueOf(x).nextProbablePrime();
            for (int j = 0; j < NUM_BIG_PRIMES / T; ++j) {
                long p = Long.parseLong(cur.toString());
                if (p > MX_VAL)
                    break;
                ret[sz++] = p;
            }
            x *= 10;
        }
        return ret;
    }

    private void brute(int i, long val, int divs, NavigableSet<Pair> cand) {
        if (i == NUM_PRIMES_FOR_GENERATING_MANY_DIVS) {
            cand.add(new Pair(divs, val));
            if (cand.size() > NUM_MANY_DIVS)
                cand.pollFirst();
            return;
        }
        for (int j = 0;; ++j) {
            brute(i + 1, val, divs * (j + 1), cand);
            if (val > CURRENT_MAXVAL_TMP / SMALL_PRIMES[i])
                break;
            val *= SMALL_PRIMES[i];
        }
    }

    private long[] calcManyDivs(long maxVal) {
        if (maxVal != MX_VAL)
            CURRENT_MAXVAL_TMP = maxVal;
        else
            CURRENT_MAXVAL_TMP = MX_VAL / 100000;

        NavigableSet<Pair> candidates = new TreeSet<>();
        brute(0, 1, 1, candidates);
        long[] ret = new long[candidates.size()];
        int i = 0;
        for (Iterator<Pair> it = candidates.iterator(); it.hasNext(); ) {
            Pair pair = it.next();
            ret[i++] = pair.value;
        }
        ret = Arrays.copyOfRange(ret, ret.length - 1000, ret.length);
        Arrays.sort(ret);
        return ret;
    }

    public long randLong(long l, long r) {
        long val = Math.abs(random.nextLong()) % (r - l + 1);
        return val + l;
    }

    private Test shuffle(Test test) {
        List<Integer> perm = new ArrayList<>();
        for (int i = 0; i < test.n; ++i)
            perm.add(i);
        Collections.shuffle(perm, random);
        for (int i = 0; i < test.m; ++i)
            for (int j = 0; j < test.indexes.get(i).size(); ++j)
                test.indexes.get(i).set(j, perm.get(test.indexes.get(i).get(j)));
        return test;
    }

    public <A, B> void synchronizedShuffle(List<? extends A> a, List <? extends B> b) {
        int seed = random.nextInt();
        Random rnda = new Random(seed);
        Random rndb = new Random(seed);
        Collections.shuffle(a, rnda);
        Collections.shuffle(b, rndb);
    }

    public int randInt(int l, int r) {
        int val = random.nextInt(r - l + 1);
        return val + l;
    }

    public long randBigPrime(long max) {
        return BIG_PRIMES[random.nextInt(upperBound(BIG_PRIMES, max) + 1)];
    }

    public long randSmallPrime(long max) {
        return SMALL_PRIMES[random.nextInt(upperBound(SMALL_PRIMES, max) + 1)];
    }

    public long randManyDivs(long max) {
        return MANY_DIVS[random.nextInt(upperBound(MANY_DIVS, max) + 1)];
    }

    public long randBigPrime() {
        return BIG_PRIMES[random.nextInt(BIG_PRIMES.length)];
    }

    public long randSmallPrime() {
        return SMALL_PRIMES[random.nextInt(SMALL_PRIMES.length)];
    }

    public long randManyDivs() {
        return MANY_DIVS[random.nextInt(SMALL_PRIMES.length)];
    }

    public static interface Generator {
        Test generate(int n, int[] ki);
    }

    public abstract class SimpleGenerator implements Generator {
        @Override
        public Test generate(int n, int[] part) {
            precalculate(n);
            Test ret = new Test(n);
            int m = part.length;

            int[] use = new int[n];
            for (int i = 0; i < m; ++i) {
                part[i] = Math.min(n, part[i]);
                newHint(part[i]);
                ret.addHint();
                for (int j = 0; j < part[i]; ++j) {
                    int id = nextIndex();
                    while (use[id] == i + 1)
                        id = nextIndex();
                    use[id] = i + 1;
                    ret.addHint(id, genValue(id));
                }
            }
            return ret;
        }

        public long genValue(int i) {
            return 0;
        }

        public void precalculate(int n) {}

        public int nextIndex() {
            return 0;
        }

        public void newHint(int hints) {
        }
    }

    public int[] getPartition(int n, int m) {
        if (m > n)
            throw new RuntimeException("components > n");
        List<Integer> perm = new ArrayList<>();
        for (int i = 1; i < n; ++i)
            perm.add(i);
        Collections.shuffle(perm, random);
        int[] partition = new int[m];
        List<Integer> prefix = perm.subList(0, m - 1);
        Collections.sort(prefix);
        prefix.add(n);
        partition[0] = prefix.get(0);
        for (int i = 0; i + 1 < prefix.size(); ++i)
            partition[i + 1] = prefix.get(i + 1) - prefix.get(i);
        return partition;
    }

    public Test buildTest(int n, int m, int sumKi, Generator generator) {
        m = Math.min(sumKi, m);
        int[] partSum = getPartition(sumKi, m);
        return shuffle(generator.generate(n, partSum));
    }

    public Test genNoRandom(int n, int m, int sumKi, long maxVal) {
        int[] partKi = getPartition(sumKi, m);
        Test ret = new Test(n);
        RandomIndexer indexer = new RandomIndexer(n);
        int[] use = new int[n];
        for (int i = 0; i < m; ++i) {
            ret.addHint();
            partKi[i] = Math.min(partKi[i], n);
            indexer.newCycle(partKi[i]);
            for (int j = 0; j < partKi[i]; ++j) {
                int id = indexer.nextIndex();
                while (use[id] == i + 1)
                    id = indexer.nextIndex();
                use[id] = i + 1;
                ret.addHint(id, randLong(1, maxVal));
            }
        }
        return ret;
    }

    private class SpecialGenerator extends SimpleGenerator {
        private int pos;

        @Override
        public int nextIndex() {
            return pos++;
        }

        @Override
        public void newHint(int hints) {
            pos = 0;
        }
    }

    private class RandomIndexer {
        private List<Integer> perm;
        private boolean isRand;
        private int pos;
        private int maxN;
        private int n;

        public RandomIndexer(int maxN) {
            this.maxN = maxN;
        }

        public int nextIndex() {
            if (isRand)
                return random.nextInt(n);
            return perm.get(pos++);
        }

        public void newCycle(int need) {
            newCycle(maxN, need);
        }

        public void newCycle(int n, int need) {
            if (need > n)
                throw new RuntimeException("need > n");
            this.n = n;
            if (need >= n / 4) {
                isRand = false;
                pos = 0;
                perm = new ArrayList<>();
                for (int i = 0; i < n; ++i)
                    perm.add(i);
                Collections.shuffle(perm, random);
            } else
                isRand = true;
        }
    }

    public Test genNoSpecialAllXOnePX(int n, long maxVal) {
        final long p = randSmallPrime(10);
        final long x = randLong(1, maxVal / p);
        final int id = random.nextInt(n);
        Test ret = buildTest(n, 1, n, new SpecialGenerator() {
            @Override
            public long genValue(int i) {
                if (i == id)
                    return x * p;
                else
                    return x;
            }
        });
        ret.addHint();
        for (int i = 0; i < n; ++i)
            ret.addHint(i, x * p);
        return ret;
    }

    public Test genNoSpecialAllPXOneX(int n, long maxVal) {
        final long p = randSmallPrime(10);
        final long x = randManyDivs(maxVal / p);
        final int id = random.nextInt(n);
        Test ret = buildTest(n, 1, n, new SpecialGenerator() {
            @Override
            public long genValue(int i) {
                if (i == id)
                    return x;
                else
                    return x * p;
            }
        });
        ret.addHint();
        for (int i = 0; i < n; ++i)
            ret.addHint(i, x * p);
        return ret;
    }

    public Test genNoManyBigPrimes(int n, int m, int sumKi, final long maxPrime) {
        return buildTest(n, m, sumKi, new SimpleGenerator() {
            private RandomIndexer indexer;
            @Override
            public void precalculate(int n) {
                indexer = new RandomIndexer(n);
            }

            @Override
            public int nextIndex() {
                return indexer.nextIndex();
            }

            @Override
            public void newHint(int hints) {
                indexer.newCycle(hints);
            }

            @Override
            public long genValue(int i) {
                return randBigPrime(maxPrime);
            }
        });
    }

    public Test genNoWithCommonDiv(int n, int m, int sumKi, long maxVal) {
        long x = randManyDivs(maxVal / 2);
        Test ret = genNoRandom(n, m, sumKi, maxVal / x);
        for (int i = 0; i < m; ++i)
            for (int j = 0; j < ret.values.get(i).size(); ++j)
                ret.values.get(i).set(j, ret.values.get(i).get(j) * x);
        return ret;
    }

    public Test genNoAddBadValues(Test yes, int cntBadVals, long maxVal) {
        for (int i = 0; i < cntBadVals; ++i) {
            yes.addHint();
            yes.addHint(random.nextInt(yes.n), randLong(1, maxVal));
        }
        return yes;
    }

    public Test genNoWithSmallChangesFromYes(Test yes, int changes, long maxVal) {
        //0 - big prime
        //1 - div other num
        //2 - random number
        final int TYPES = 3;
        int sumKi = yes.getSumKi();
        int curSum = 0;
        double probably = Math.min(1.0, changes * 1.0 / sumKi);
        for (int i = 0; i < yes.m && changes > 0; ++i) {
            int sz = yes.indexes.get(i).size();
            long mn = maxVal;
            ArrayList <Long> values = yes.values.get(i);
            for (int j = 0; j < sz; ++j) {
                double val = random.nextDouble();
                if (val < probably || sumKi - curSum <= changes) {
                    int type = random.nextInt(TYPES);
                    if (type == 0)
                        values.set(j, randBigPrime(maxVal));
                    else if (type == 2)
                        values.set(j, randLong(1, maxVal));
                    else {
                        long num = (j > 0 ? values.get(j - 1) : j + 1 < sz ? values.get(j + 1) : -1);
                        if (num == -1)
                            continue;
                        if (2 * num <= maxVal)
                            values.set(j, randLong(1, maxVal / num) * num);
                        else
                            values.set(j, randLong(1, maxVal / mn) * mn);
                    }
                    --changes;
                }
                mn = Math.min(mn, yes.values.get(i).get(j));
                ++curSum;
            }
            synchronizedShuffle(yes.indexes.get(i), yes.values.get(i));
        }
        return yes;
    }

    public Test genNoOneIndex(int n, int m, long maxVal) {
        Test ret = genYesOneIndex(n, m, maxVal);
        int id = ret.indexes.get(0).get(0);
        int id2 = random.nextInt(n);

        ret.addHint();
        ret.addHint(id2, randLong(1, maxVal));

        ret.addHint();
        ret.addHint(id, randLong(1, maxVal));
        ret.addHint(id2, randLong(1, maxVal));
        return ret;
    }

    private abstract class AbstractYesGenerator extends SimpleGenerator {
        protected long[] b;
        protected long t;
        private RandomIndexer indexer;
        protected long maxValOfBi;

        public AbstractYesGenerator(long maxVal) {
            this.maxValOfBi = maxVal;
        }

        @Override
        public void precalculate(int n) {
            b = new long[n];
            for (int i = 0; i < n; ++i)
                b[i] = genBi();
            Arrays.sort(b);
            indexer = new RandomIndexer(n);
        }

        @Override
        public void newHint(int hints) {
            t = genT(hints);
            int rights = upperBound(b, maxValOfBi / t);
            indexer.newCycle(rights + 1, hints);
        }

        @Override
        public long genValue(int i) {
            return t * b[i];
        }

        @Override
        public int nextIndex() {
            return indexer.nextIndex();
        }

        public long genT(int hints) {
            return randLong(1, maxValOfBi / b[hints - 1]);
        }

        public abstract long genBi();
    }

    public Test genYesOneIndex(int n, int m, long maxVal) {
        return buildTest(n, m, m, new AbstractYesGenerator(maxVal) {
            @Override
            public long genBi() {
                long bi = randLong(1, 100);
                return bi;
            }

            @Override
            public int nextIndex() {
                return 0;
            }
        });
    }

    public Test genYesCommonDiv(int n, int m, int sumKi, long maxVal, long commonDivisor) {
        Test ret = buildTest(n, m, sumKi, new AbstractYesGenerator(maxVal / commonDivisor) {
            @Override
            public long genBi() {
                return randLong(1, this.maxValOfBi);
            }
        });

        for (int i = 0; i < ret.m; ++i)
            for (int j = 0; j < ret.values.get(i).size(); ++j)
                ret.values.get(i).set(j, ret.values.get(i).get(j) * commonDivisor);
        return ret;
    }

    public Test genYesManyDivsBi(int n, int m, int sumKi, long maxVal) {
        return buildTest(n, m, sumKi, new AbstractYesGenerator(maxVal) {
            @Override
            public long genBi() {
                return randManyDivs(this.maxValOfBi);
            }
        });
    }

    public Test genYesBiBigPrimes(int n, int m, int sumKi, long maxVal, final long maxPrime) {
        return buildTest(n, m, sumKi, new AbstractYesGenerator(maxVal) {
            @Override
            public long genBi() {
                return randBigPrime(maxPrime);
            }

            @Override
            public long genT(int hints) {
                long maxT = this.maxValOfBi / this.b[hints - 1];
                if (maxT <= MIN_BIG_PRIME)
                    return SMALL_PRIMES[upperBound(SMALL_PRIMES, maxT)];
                return randBigPrime(maxT);
            }
        });
    }

    public Test genYesBiSmallPrimes(int n, int m, int sumKi, long maxVal, final long lowerPrime) {
        final int left = upperBound(SMALL_PRIMES, lowerPrime);
        return buildTest(n, m, sumKi, new AbstractYesGenerator(maxVal) {
            @Override
            public long genT(int hints) {
                long maxT = this.maxValOfBi / this.b[hints - 1];
                if (maxT <= MAX_SMALL_PRIME)
                    return SMALL_PRIMES[upperBound(SMALL_PRIMES, maxT)];
                return randBigPrime(maxT);
            }

            @Override
            public long genBi() {
                return SMALL_PRIMES[randInt(left, SMALL_PRIMES.length - 1)];
            }
        });
    }

    public Test genTrickTest(int n) {
        Test ret = new Test(n);
        for (int i = 0; i < n - 2; i++) {
            ret.addHint();
            ret.addHint(i, 2);
            ret.addHint(i + 1, 2);
        }
        ret.addHint();
        ret.addHint(n - 2, 2);
        ret.addHint(n - 1, 1);
        return ret;
    }

    public void genPreliminary() throws IOException {
        Test test1 = new Test(3);
        test1.addHint();
        test1.addHint(0, 6);
        test1.addHint(1, 9);
        test1.addHint();
        test1.addHint(0, 8);
        write(test1, "preliminary", "subtask0", 0, "sample1");

        Test test2 = new Test(5);
        test2.addHint();
        test2.addHint(0, 4);
        test2.addHint(1, 6);
        test2.addHint();
        test2.addHint(1, 12);
        test2.addHint(2, 8);
        test2.addHint();
        test2.addHint(1, 9);
        test2.addHint(2, 6);
        test2.addHint(3, 3);
        write(test2, "preliminary", "subtask0", 0, "sample2");

        Test test3 = new Test(3);
        test3.addHint();
        test3.addHint(0, 6);
        test3.addHint(1, 9);
        test3.addHint();
        test3.addHint(0, 9);
        test3.addHint(1, 6);
        write(test3, "preliminary", "subtask0", 0, "sample3");

        testCount = 0;
    }

    public void genHandleTests(int mx, int points) throws IOException {
    }

    public void genSubtask(long maxVal, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + Integer.toString(subtaskCount);
        int bound = Math.min(MAX_SMALL_PRIME, (int) Math.sqrt(maxVal));
        SMALL_PRIMES = calcSmallPrimes(bound);
        if (maxVal == LIMITS[1])
            BIG_PRIMES = calcSmallPrimes((int) maxVal);
        else
            BIG_PRIMES = calcBigPrimes();
        MANY_DIVS = calcManyDivs(maxVal);

        write(genTrickTest(MX_N), "tests", subtask, 0, "trick test n = MX_N");

        write(genYesCommonDiv(MX_N, 10, MX_SUM_KI, maxVal, SMALL_PRIMES[SMALL_PRIMES.length - 1]), "tests", subtask, 0, "yes with common div = the max small prime, n=10^5/2, m=10^5/2, sumKi=10^6/2");
        write(genYesCommonDiv(MX_N, 10000, MX_SUM_KI / 5, maxVal, randSmallPrime()), "tests", subtask, 0, "yes with common = rand small prime, n=10^5, m=100, sumKi=10^6");

        write(genYesManyDivsBi(MX_N, MX_M, MX_SUM_KI, maxVal), "tests", subtask, 0, "yes bi with many divs, n=10^5/2, m=10^5/2, sum=10^6/2");
        write(genYesManyDivsBi(MX_N, 100, MX_SUM_KI / 2, maxVal), "tests", subtask, 0, "yes bi with many divs, n=10^5/2, m=10^5/2, sum=10^6/2");
        write(genYesManyDivsBi(MX_N, 10, MX_SUM_KI, maxVal), "tests", subtask, 0, "yes bi with many divs, n=10^5, m=100, sum=10^6");
        write(genYesManyDivsBi(MX_N, 1, MX_N, maxVal), "tests", subtask, 0, "yes bi with many divs, n=10^5, m=100, sum=10^6");

        write(genYesBiBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, maxVal), "tests", subtask, 0, "yes bi is big prime, n=10^5/2, m=10^5/2, sum=10^6/2");
        if (maxVal > LIMITS[1]) {
            write(genYesBiBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, maxVal / 100000), "tests", subtask, 0, "yes bi is big prime, n=10^5/2, m=10^5/2, sum=10^6/2");
            write(genYesBiBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, maxVal / 100000000), "tests", subtask, 0, "yes bi is big prime, n=10^5/2, m=10^5/2, sum=10^6/2");
            write(genYesBiBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, maxVal / 100000000), "tests", subtask, 0, "yes bi is big prime, n=10^5/2, m=10^5/2, sum=10^6/2");
            write(genYesBiBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, maxVal / 100000000), "tests", subtask, 0, "yes bi is big prime, n=10^5/2, m=10^5/2, sum=10^6/2");
        }

        write(genYesBiSmallPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, bound / 10), "tests", subtask, 0, "yes bi is small prime, n=10^5/2, m=10^5/2, sum=10^6/2");
        write(genYesBiSmallPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, bound / 100), "tests", subtask, 0, "yes bi is small prime, n=10^5/2, m=10^5/2, sum=10^6/2");
        if (maxVal > LIMITS[1]) {
            write(genYesBiSmallPrimes(MX_N, MX_M, MX_SUM_KI, maxVal, bound / 1000), "tests", subtask, 0, "yes bi is small prime, n=10^5/2, m=10^5/2, sum=10^6/2");
        }

        write(genYesOneIndex(MX_N, MX_M, maxVal), "tests", subtask, 0, "yes one index hints, n = 10^5, m=10^5");
        write(genYesOneIndex(MX_N / 2, MX_M, maxVal), "tests", subtask, 0, "yes one index hints, n = 10^5/2, m=10^5");

        System.out.println("TESTS YES FINISHED!");

        //NO tests
        write(genNoOneIndex(MX_N / 2, MX_M - 2, maxVal), "tests", subtask, 0, "no one index hints, n = 10^5, m=10^5");
        write(genNoOneIndex(MX_N, MX_M - 2, maxVal), "tests", subtask, 0, "no one index hints, n = 10^5, m=10^5");

        write(genNoManyBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal), "tests", subtask, 0, "no many big primes, n = 10^5, m=10^5, sumKi=10^6");
        if (maxVal > LIMITS[1]) {
            write(genNoManyBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal / 10000000), "tests", subtask, 0, "no many big primes, n = 10^5, m=10^5, sumKi=10^6");
            write(genNoManyBigPrimes(MX_N, MX_M, MX_SUM_KI, maxVal / 100000000), "tests", subtask, 0, "no many big primes, n = 10^5/2, m=10^5");
        }

        write(genNoRandom(MX_N, MX_M, MX_SUM_KI, maxVal), "tests", subtask, 0, "no full random, n = 10^5, m=10^5, sumKi=10^6");
        write(genNoRandom(MX_N, MX_M / 2, MX_SUM_KI / 2, maxVal), "tests", subtask, 0, "no full random, n = 10^5, m=10^5, sumKi=10^6");

        write(genNoSpecialAllPXOneX(MX_N, maxVal), "tests", subtask, 0, "no special - one x, other p*x, n = 10^5, m = 1");
        write(genNoSpecialAllXOnePX(MX_N, maxVal), "tests", subtask, 0, "no special - one p*x, other x, n = 10^5, m = 1");

        write(genNoWithCommonDiv(MX_N, 10, MX_SUM_KI, maxVal), "tests", subtask, 0, "no with common div, n = 10^5/2, m=10^5, sumKi=10^6");
        write(genNoWithCommonDiv(MX_N, MX_M / 2, MX_SUM_KI / 5, maxVal), "tests", subtask, 0, "no with common div, n = 10^5/2, m=10^5/2, sumKi=10^6/5");

        write(genNoAddBadValues(genYesCommonDiv(MX_N, 10, MX_SUM_KI - 20, maxVal, SMALL_PRIMES[SMALL_PRIMES.length - 1]), 3, maxVal), "tests", subtask, 0, "no with common div = the max small prime, n=10^5/2, m=10^5/2, sumKi=10^6/2 with bad extra information");
        write(genNoWithSmallChangesFromYes(genYesCommonDiv(MX_N, 100, MX_SUM_KI - 10, maxVal, SMALL_PRIMES[SMALL_PRIMES.length - 1]), 5, maxVal), "tests", subtask, 0, "no with common div = the max small prime, n=10^5/2, m=10^5/2, sumKi=10^6/2 with small changes");

        write(genNoAddBadValues(genYesManyDivsBi(MX_N, MX_M - 100, MX_SUM_KI - 100, maxVal), 100, maxVal), "tests", subtask, 0, "no bi with many divs, n=10^5/2, m=10^5/2, sum=10^6/2 with bad extra information");
        write(genNoWithSmallChangesFromYes(genYesManyDivsBi(MX_N, MX_M / 2, MX_SUM_KI - 5, maxVal), 2, maxVal), "tests", subtask, 0, "no bi with many divs, n=10^5, m=100, sum=10^6 with small changes");

        if (maxVal > LIMITS[1]) {
            write(genYesCommonDiv(MX_N, 100, MX_SUM_KI, maxVal, SMALL_PRIMES[SMALL_PRIMES.length - 1]), "tests", subtask, 0, "yes with common div = the max small prime, n=10^5, m=100, sumKi=MAX");
            write(genYesCommonDiv(MX_N, 10, MX_SUM_KI, maxVal, randBigPrime(1000000000)), "tests", subtask, 0, "yes with common div = big prime, n=10^5/2, m=10^5, sumKi=MAX");
            write(genYesCommonDiv(MX_N, 100, MX_SUM_KI, maxVal, randBigPrime(1000000000)), "tests", subtask, 0, "yes with common div = big prime, n=10^5/2, m=10^5, sumKi=MAX");
            write(genYesCommonDiv(MX_N, 10, MX_SUM_KI, maxVal, randBigPrime(10000000000L)), "tests", subtask, 0, "yes with common div = big prime, n=10^5/2, m=10^5, sumKi=MAX");
        }
        System.out.println("GROUP FINISHED!");

        testsDesc.println(String.format("comment_subtask%d=\"xij &lt;= %s\"", subtaskCount, Long.toString(maxVal)));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(LIMITS[1], 50);
        genSubtask(LIMITS[2], 50);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
