#!/usr/bin/env perl

use strict;
use warnings;

use Term::ANSIColor;
use Cwd qw/getcwd/;
use File::Copy;
use File::Basename;

my $TL = 3;
my $problem = basename getcwd;
my $author = shift;
my $source_name = $problem . "_" . $author;

die colored("Usage ./check_solution.pl author", 'red') if !defined($author);

my $lang = shift;
`g++ -O3 $source_name.cpp -o $source_name -std=c++11`;
if (!defined($lang) && $? == 0) {
	$lang = "cpp";
} elsif (!defined($lang)) {
	`javac $source_name.java`;
	if ($? == 0) {
		$lang = "java";
	} else {
		die colored("Cannot compile $source_name", 'red')
	}
}

open F, "<tests.desc";

my @cur;
my %cost;
my %subtask;
while (<F>) {
    if (m#(tests/\d+)#) {
        push @cur, $1;
    } elsif (/bonus_subtask(\d)=(\d+)/) {
        $cost{$1} = $2;
        for my $x (@cur) {
            $subtask{$x} = $1;
        }
        @cur = ();
    }
}

close F;

my $score = 100;
my %used;
sub badans {
    my $test = shift;
    my $sg = $subtask{$test};
    if (defined($sg) && !defined($used{$sg})) {
        $used{$sg} = 1;
        $score -= $cost{$sg};
        print colored("Failed subtask $sg (-$cost{$sg})\n", "yellow");
    }
}

sub test {
    my $last = undef;
    for my $test (glob shift) {
        next if ($test =~ /\.a$/);
        my $sg = $subtask{$test};
        if (defined($sg) && (!defined($last) || $last != $sg)) {
            print colored("=============== Testing subtask $sg =============\n", "yellow");
            $last = $sg;
        }
        copy "$test", "$problem.in";
        if ($lang eq "cpp") {
            `bash -c '{ time timeout $TL ./$source_name <$problem.in >$problem.out ; } 2>log.txt'`;
        } else {
            `bash -c '{ time timeout $TL java $source_name <$problem.in >$problem.out ; } 2>log.txt'`;
        }
        my $res = $?;
        open F, "<log.txt";
        my $time = "";
        $time .= $_ while <F>;
        close F;
        $time =~ /real\s(.*)/;
        $time = $1;
        if ($res == 0) {
            my $output = `java -Xmx256M -Xss64M -cp ../../../lib/testlib4j.jar:. ru.ifmo.testlib.CheckerFramework Check $test $problem.out $test.a`;
            if ($? == 0) {
                print colored("OK on test $test in $time .\n", 'green');
            } else {
                print colored("WA/PE on test $test in $time s.\n$output", 'red');
                badans $test;
            }
        } else {
            print colored("TL/RE on test $test in $time s.\n", 'red');
            badans $test;
        }
    }
}

test "preliminary/*";
test "tests/*";
print colored("=============== Total score $score =============\n", "yellow");
