import java.io.*;
import java.math.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    Random random = new Random(787788);
    int MAX_NK = 100_000;
    int MX = 1_000_000_000;
    int MOD = 1_000_000_007;
    
    int[][] samples = new int[][]{{1, 2, 0, 2, 0}};
    int[][] handTests = new int[][]{{0, 1, 0, 2, 2}};

    void write(int[] a, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(a.length);
        for (int i = 0; i < a.length; i++) {
            out.print(a[i]);
            if (i + 1 == a.length) {
                out.println();
            } else {
                out.print(" ");
            }
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }

    void genPreliminary() throws IOException {
        for (int i = 0; i < samples.length; i++) {
            write(samples[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

    void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = random.nextInt(i + 1);
            int o = a[j];
            a[j] = a[i];
            a[i] = o;
        }
    }

    void genRandom(int n) throws IOException {
        int a[] = new int[n];
            
        while (true) {        
            for (int i = 0; i < n; i++) {
                a[random.nextInt(n)]++;
            }
            int mx = 0;
            for (int i = 0; i < n; i++) {
                mx = Math.max(mx, a[i]);
            }
            if (mx < n) {
                break;
            }
        }

        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d\"", n));
    }             
    
    void genOnes(int n) throws IOException {
        int a[] = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = 1;
        }
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"all ones, n=%d\"", n));
    }

    void genOneMax(int n) throws IOException {
        int a[] = new int[n];
        a[0] = n - 1;
        a[1] = 1;
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"One maximum, n=%d\"", n));
    }

    void genArithm(int n) throws IOException {
        int a[] = new int[n];
        int sum = n;
        for (int i = 0; i < n; i++) {
            a[i] = Math.min(i, sum);
            sum -= a[i];
        }
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"natural numbers, n=%d\"", n));
    }
   
    void genRandStrange(int n) throws IOException {
        int a[] = new int[n];
        int sum = n;
        for (int i = 0; i < n; i++) {
            a[i] = random.nextInt(sum + 1);
            sum -= a[i];
        }
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d\"", n));
    }
 
    void genManyZeroes(int n) throws IOException {
        int a[] = new int[n];
        int sum = n;
        for (int i = 0; i < n; i++) {
            int ost = random.nextInt(sum / 20 + 1) + sum / 20;
            a[i] = sum - ost; 
            sum -= a[i];
        }
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"many zeroes, n=%d\"", n));
    }

    void genExpProb(int n) throws IOException {
        int a[] = new int[n];
        int sum = n;
        for (int i = 0; i < n; i++) {
            while (sum > 0 && random.nextInt(2) == 0) {
                sum--;
                a[i]++;
            }
        }
        a[n - 1] += sum;
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"exp probability, n=%d\"", n));
    }

    void genAntiRandom(int n) throws IOException {
        int a[] = new int[n];
        a[0] = n - 2;
        a[1] = 2;
        shuffle(a);
        write(a, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"anti rand, n=%d\"", n));
    }

    void genHandTests() throws IOException {
        for (int i = 0; i < handTests.length; i++) {
            write(handTests[i], "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"small hand test, n=%d\"", handTests[i].length));
        }
    }
 
    void genSubtask(int n, int points) throws IOException {
        ++subtaskCount;

        genHandTests();
        genOnes(3);
        genOnes(n);
        genOnes(n - 1); 
        genOneMax(n);
        for (int i = 0; i < 1 + subtaskCount; i++) {
            genRandom(n - random.nextInt(n / 3));
        }
        genRandom(n);
        genArithm(n);
        int ss = (int)(Math.sqrt(2 * n));
        genArithm(ss * (ss - 1) / 2);
        for (int i = 0; i < 1 + subtaskCount; i++) {
            genRandStrange(n);
        }
        for (int i = 0; i < (subtaskCount >= 2 ? 3 : 0); i++) {
            genManyZeroes(n);
            genExpProb(n);
        }
        if (subtaskCount >= 2) {
            genAntiRandom(n);
        }

        testsDesc.println(String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));       
    }

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(15, 20);
        genSubtask(300, 40);
        genSubtask(100_000, 40);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
